@@ -104,7 +104,7 @@ nimbleModel <- function(code,
     md$setupModel(code=code, constants=constants, dimensions=dimensions, inits = inits,
                   data = data, userEnv = userEnv, debug=debug)
     if(!returnModel) return(md)
-    if(nimbleOptions("enableModelMacros")){
+    if(nimbleOptions("enableMacros")){
       # Do this stuff if macros are enabled:
       # Update constants in case any new ones were created by macros
       newConstants <- md$constantsList[!names(md$constantsList) %in% names(constants)]
