@@ -1241,8 +1241,7 @@ nimRound <- round
 #'   (but not required) in "nlminb" or (optionally) user-provided methods.
 #' @param ... IGNORED
 #' @param method The method to be used. See `Details` section of
-#'   \code{\link{optim}}. One of: "Nelder-Mead", "BFGS", "CG", "L-BFGS-B", or
-#'   "nlminb". Note that the R methods "SANN", "Brent" are not supported. It is
+#'   \code{\link{optim}}. One of: "Nelder-Mead", "BFGS", "CG", "L-BFGS-B", "nlminb", or "bobyqa". Note that the R methods "SANN", "Brent" are not supported. It is
 #'   also possible to provide a new method; see details.
 #' @param lower Vector or scalar of lower bounds for parameters.
 #' @param upper Vector or scalar of upper bounds for parameters.
@@ -1268,7 +1267,10 @@ nimRound <- round
 #'
 #' For \code{method="nlminb"}, a compiled nimbleFunction will run R's
 #' \code{nlminb} directly in R, with \code{fn}, \code{gr} (if provided) and
-#' \code{he} (if provided) that call back into compiled code.
+#' \code{he} (if provided) that call back into compiled code. For
+#' \code{method="bobyqa"}, a compiled nimbleFunction will run R's
+#' \code{nloptr::bobyqa} directly in R, with \code{fn} that calls back
+#' into compiled code.
 #'
 #' An experimental feature is the capability to provide one's own optimization
 #' method in R and register it for use by \code{nimOptim}. One must write a
@@ -1279,12 +1281,15 @@ nimRound <- round
 #' \code{hessian} (which may be NULL). If \code{hessian=TRUE} but the function
 #' does not return a matrix in the \code{hessian} element of its return list,
 #' \code{nimOptim} will fill in that element using finite differences of the
-#' gradient.
+#' gradient. In general the function will be a wrapper around the actual
+#' R optimization function.
 #'
 #' The \code{control} list passed from a nimbleFunction to the
 #' optimization function will include a minimum of options, including
-#' \code{abstol}, \code{reltol}, \code{maxit}, and \code{trace}. Other options
-#' for a specific method may be set within the custom optimization function but
+#' \code{abstol}, \code{reltol}, \code{maxit}, and \code{trace}. This means
+#' that the user's R (wrapper) function must map between those minimum options
+#' and the equivalent inputs to the optimization function. Other options
+#' for a specific method may be set within the R (wrapper) function but
 #' cannot be passed from \code{nimOptim}.
 #'
 #'  The elements \code{parscale} and \code{fnscale} in \code{control} are used in
@@ -1293,11 +1298,12 @@ nimRound <- round
 #' for it to minimize \code{fn(par)/fnscale} in the parameter space
 #' \code{par/parscale}.
 #'
-#' An optimizer \code{fun} may be registered by
-#' \code{nimOptimMethod("method_name", fun)}, and then "\code{method_name}" can
-#' be used as the \code{method} argument to \code{nimOptim} to use \code{fun}.
-#' An optimizer may be found by \code{nimOptimMethod("method_name")} and may be
-#' removed by \code{nimOptimMethod("method_name", NULL)}.
+#' To use the optimizer with \code{nimOptim}, an optimizer \code{fun} must be
+#' registered by \code{nimOptimMethod("method_name", fun)}, and then
+#' "\code{method_name}" can be used as the \code{method} argument to
+#' \code{nimOptim} to use \code{fun}. An optimizer may be found by
+#' \code{nimOptimMethod("method_name")} and may be removed by
+#' \code{nimOptimMethod("method_name", NULL)}.
 #'
 #' Support for \code{method="nlminb"} is provided in this way, and can be
 #' studied as an example via \code{nimOptimMethod("nlminb")}.
@@ -1418,7 +1424,7 @@ nimOptim <- function(par, fn, gr = "NULL", he = "NULL", ..., method = "Nelder-Me
 
 # nimble_custom_optim_ function is called only from C++
 custom_optim <- function(method, par, lower, upper, control,
-                         hessian, use_gr, use_he, extptr,...) {
+                         hessian, use_gr, use_he, extptr, ...) {
     
   fnsym <- nimbleUserNamespace$sessionSpecificDll$CALL_NimOptimProblem_fn # getNativeSymbolInfo("CALL_NimOptimProblem_fn")
   fn <- function(p) eval(call('.Call', fnsym, p, extptr))
@@ -1452,14 +1458,23 @@ custom_optim_inner <- function(method, par, fn, gr, he, lower, upper, control,
   optimizer <- nimOptimMethod(method)
   if(is.null(optimizer)) stop("optimizer ", method, " not found. See help(nimOptimMethod).")
   resultRaw <- try(optimizer(par, fn, gr = gr, he = he, lower = lower,
-                             upper = upper, control = control, hessian=hessian))
+                             upper = upper, control = control, hessian=hessian),
+                             silent = TRUE)
   ##  }
-  if(inherits(resultRaw, "try-error")) stop("problem with optimizer ", method)
-
   result <- list()
-  for(name in c("par", "value", "convergence", "counts", "message", "hessian")) {
-    result[[name]] <- resultRaw[[name]]
-    attributes(result[[name]]) <- NULL
+  if(inherits(resultRaw, "try-error")) {
+    # stop("problem with optimizer ", method)
+    result[['par']] = par
+    result[['value']] = NaN
+    result[['convergence']] = 1
+    result[['counts']] = 0
+    result[['message']] = ''
+    result[['hessian']] = matrix(nrow = 0, ncol = 0)
+  } else {
+    for(name in c("par", "value", "convergence", "counts", "message", "hessian")) {
+      result[[name]] <- resultRaw[[name]]
+      attributes(result[[name]]) <- NULL
+    }
   }
   result
 }
