@@ -36,6 +36,15 @@ virtualNFprocessing <- setRefClass('virtualNFprocessing',
                     name <<- className
                 }
                 origMethods <<- nf_getMethodList(nfGenerator)
+                origMethodNames <- names(origMethods)
+                conflictedNames <- origMethodNames %in% c(names(nimble:::cppOutputCalls), names(nimble:::specificCallReplacements),
+                                                          binaryOperators, unaryOperators,
+                                                          reductionUnaryOperators, matrixSquareReductionOperators,
+                                                          reductionBinaryOperators, nonNativeEigenCalls,
+                                                          matrixFlipOperators, matrixSquareOperators,
+                                                          nimbleListReturningOperators, matrixSolveOperators)
+                if(any(conflictedNames)) 
+                    stop("The name of the nimbleFunction method `", paste0(origMethodNames[conflictedNames], collapse = '`,'), "` conflicts with a function in the NIMBLE language (DSL); please use a different name")
                 RCfunProcs <<- list()
                 for(i in seq_along(origMethods)) {
                     RCname <- names(origMethods)[i]
