@@ -185,6 +185,42 @@ test_that('AD works from list with an ignore entry buildDerivs with setup = TRUE
   nim_expect_equal(Rans$hessian, Cans$hessian, RCrelTol2)
 })
 
+test_that('AD works with ADbreak with setup = TRUE', {
+  adfun <- nimbleFunction(
+    setup = TRUE,
+    run = function(x = double(1)) {
+      ans <- 0
+      for(i in 1:length(x))
+        ans <- ans + exp(x[i])
+      ans1 <- ADbreak(ans) # ans1 is CppAD::AD but assigned only value
+      ans2 <- ADbreak(ans) # ans2 is double
+      myint <- 0L
+      myint <- ADbreak(ans) # assignment to integer also requires ADbreak
+      ans <- ans2          # can assign from double to CppAD::AD
+      return(ans)
+      returnType(double())
+    },
+    methods = list(
+      derivsRun = function(x = double(1)) {
+        ans <- nimDerivs(run(x), wrt = 1:length(x), order = 0:2)
+        return(ans)
+        returnType(ADNimbleList())
+      }),
+    buildDerivs = list(run = list(ignore = c('i', 'ans2')))
+  )
+  temporarilyAssignInGlobalEnv(adfun)
+  Radfun1 <- adfun()
+  Cadfun1 <- compileNimble(Radfun1)
+  xRec <- c(.3, -1.2, 2.1)
+  xTest <- c(-.4, 1.9, -0.8)
+  Rans <- Radfun1$derivsRun(xTest) # does not respect ADbreak
+  CansRec <- Cadfun1$derivsRun(xRec) # xRec gets baked in due to ADbreak
+  Cans <- Cadfun1$derivsRun(xTest)
+  nim_expect_equal(CansRec$value, Cans$value, RCrelTol0)
+  nim_expect_equal(Cans$jacobian, matrix(rep(0, 3), ncol = 3), RCrelTol1)
+  nim_expect_equal(Cans$hessian, array(rep(0, 9), dim=c(3,3,1)), RCrelTol2)
+})
+
 test_that('AD works with calls to other methods and RCfunctions', {
   adhelper <- nimbleFunction(
     run = function(x = double(1)) {
@@ -227,6 +263,144 @@ test_that('AD works with calls to other methods and RCfunctions', {
   nim_expect_equal(Rans$hessian, Cans$hessian, RCrelTol2)
 })
 
+test_that('AD works with calls to methods across different nimbleFunction objects (1)', {
+  adhelper <- nimbleFunction(
+    setup=TRUE,
+    run = function(x = double(1)) {
+      ans <- sum(exp(x))
+      return(ans)
+      returnType(double(0))
+    },
+    buildDerivs='run'
+  )
+#  adhelper1 <- adhelper
+
+  adfun <- nimbleFunction(
+    setup = function() {adhelper1 <- adhelper()},
+    run = function(x = double(1)) {
+      ans <- 2*mysqrt(adhelper1$run(x))
+      return(ans)
+      returnType(double())
+    },
+    methods = list(
+      mysqrt = function(x = double()) {
+        return(sqrt(x))
+        returnType(double())
+      },
+      derivsRun = function(x = double(1)) {
+        ans <- nimDerivs(run(x), wrt = 1:length(x), order = 0:2)
+        return(ans)
+        returnType(ADNimbleList())
+      }),
+    buildDerivs = c('run', 'mysqrt')
+  )
+  Radfun1 <- adfun()
+  Cadfun1 <- compileNimble(Radfun1)
+  xRec <- c(.3, -1.2, 2.1)
+  xTest <- c(-.4, 1.9, -0.8)
+  Rans <- Radfun1$derivsRun(xTest)
+  Cadfun1$derivsRun(xRec)
+  Cans <- Cadfun1$derivsRun(xTest)
+  nim_expect_equal(Rans$value, Cans$value, RCrelTol0)
+  nim_expect_equal(Rans$jacobian, Cans$jacobian, RCrelTol1)
+  nim_expect_equal(Rans$hessian, Cans$hessian, RCrelTol2)
+})
+
+test_that('AD works with calls to methods across different nimbleFunction objects (2)', {
+  adhelper <- nimbleFunction(
+    setup=TRUE,
+    run = function(x = double(1)) {
+      ans <- sum(exp(x))
+      return(ans)
+      returnType(double(0))
+    },
+    buildDerivs='run'
+  )
+  adhelper1 <- adhelper()
+
+  adfun <- nimbleFunction(
+    setup = function(adhelper1) {},
+    run = function(x = double(1)) {
+      ans <- 2*mysqrt(adhelper1$run(x))
+      return(ans)
+      returnType(double())
+    },
+    methods = list(
+      mysqrt = function(x = double()) {
+        return(sqrt(x))
+        returnType(double())
+      },
+      derivsRun = function(x = double(1)) {
+        ans <- nimDerivs(run(x), wrt = 1:length(x), order = 0:2)
+        return(ans)
+        returnType(ADNimbleList())
+      }),
+    buildDerivs = c('run', 'mysqrt')
+  )
+  Radfun1 <- adfun(adhelper1)
+  Cadfun1 <- compileNimble(Radfun1)
+  xRec <- c(.3, -1.2, 2.1)
+  xTest <- c(-.4, 1.9, -0.8)
+  Rans <- Radfun1$derivsRun(xTest)
+  Cadfun1$derivsRun(xRec)
+  Cans <- Cadfun1$derivsRun(xTest)
+  nim_expect_equal(Rans$value, Cans$value, RCrelTol0)
+  nim_expect_equal(Rans$jacobian, Cans$jacobian, RCrelTol1)
+  nim_expect_equal(Rans$hessian, Cans$hessian, RCrelTol2)
+})
+
+test_that('AD works with calls to methods across different nimbleFunction objects with double taping (1)', {
+  adhelper <- nimbleFunction(
+    setup=TRUE,
+    methods = list(
+      run_helper = function(x = double(1)) {
+        ans <- sum(exp(x))
+        return(ans)
+        returnType(double(0))
+      },
+      derivsRun_helper = function(x = double(1)) {
+        ans <- nimDerivs(run_helper(x), wrt = 1:length(x), order = 1)
+        jac <- ans$jacobian[1,]
+        jac2 <- sum(jac * jac) # make it positive for sqrt'ing
+        return(jac2)
+        returnType(double(0))
+      }
+    ),
+    buildDerivs=c('run_helper', 'derivsRun_helper')
+  )
+  adhelper1 <- adhelper()
+
+  adfun <- nimbleFunction(
+    setup = function(adhelper1) {},
+    run = function(x = double(1)) {
+      ans <- 2*mysqrt(adhelper1$derivsRun_helper(x))
+      return(ans)
+      returnType(double())
+    },
+    methods = list(
+      mysqrt = function(x = double()) {
+        return(sqrt(x))
+        returnType(double())
+      },
+      derivsRun = function(x = double(1)) {
+        ans <- nimDerivs(run(x), wrt = 1:length(x), order = 0:2)
+        return(ans)
+        returnType(ADNimbleList())
+      }),
+    buildDerivs = c('run', 'mysqrt')
+  )
+  Radfun1 <- adfun(adhelper1)
+  Cadfun1 <- compileNimble(Radfun1)
+  xRec <- c(.3, -1.2, 2.1)
+  xTest <- c(-.4, 1.9, -0.8)
+  Rans <- Radfun1$derivsRun(xTest)
+  Cadfun1$derivsRun(xRec)
+  Cans <- Cadfun1$derivsRun(xTest)
+  nim_expect_equal(Rans$value, Cans$value, 1e-10)
+  nim_expect_equal(Rans$jacobian, Cans$jacobian, 1e-3)
+#  nim_expect_equal(Rans$hessian, Cans$hessian, RCrelTol2) # uncompiled looks fairly rough. I think it's ok but won't try to test.
+})
+
 test_that('AD works with calls to other methods and RCfunctions specified in a list', {
   adhelper <- nimbleFunction(
     run = function(x = double(1)) {
@@ -327,7 +501,7 @@ test_that('AD works with model$calulate in a method', {
     }),
     buildDerivs = TRUE
   )
-  temporarilyAssignInGlobalEnv(m)
+#  temporarilyAssignInGlobalEnv(m)
   
   adfun <- nimbleFunction(
     setup = function(model, nodes) {
@@ -344,7 +518,7 @@ test_that('AD works with model$calulate in a method', {
     methods = list(
       derivsRun = function(x = double(1)) {
         ans <- nimDerivs(run(x), wrt = 1:length(x), order = 0:2,
-                         model = m, updateNodes = updateNodes)
+                         model = model, updateNodes = updateNodes)
         return(ans)
         returnType(ADNimbleList())
       }),
@@ -411,7 +585,7 @@ test_that('AD works with a model compiled in one call', {
     }),
     buildDerivs = TRUE
   )
-  temporarilyAssignInGlobalEnv(m)
+  # temporarilyAssignInGlobalEnv(m)
   
   adfun <- nimbleFunction(
     setup = function(model, nodes) {
@@ -428,7 +602,7 @@ test_that('AD works with a model compiled in one call', {
     methods = list(
       derivsRun = function(x = double(1)) {
         ans <- nimDerivs(run(x), wrt = 1:length(x), order = 0:2,
-                         model = m, updateNodes = updateNodes)
+                         model = model, updateNodes = updateNodes)
         return(ans)
         returnType(ADNimbleList())
       }),
@@ -448,7 +622,7 @@ test_that('AD works with a model compiled in one call', {
   nim_expect_equal(Rans$hessian, Cans$hessian, RCrelTol2)
 })
 
-test_that('AD works with a model calling a custom dist and custom fun', {
+test_that('AD works with a model calling a user-defined distribution and user-defined function', {
   foo <- nimbleFunction(
     run = function(x = double()) {
       return(exp(x))
@@ -484,7 +658,7 @@ test_that('AD works with a model calling a custom dist and custom fun', {
     inits = list(a = c(.1, .2, .3), x = c(.2, .3, .4)),
     buildDerivs = TRUE
   )
-  temporarilyAssignInGlobalEnv(m)
+  # temporarilyAssignInGlobalEnv(m)
   
   adfun <- nimbleFunction(
     setup = function(model, wrt, calc) {
@@ -501,7 +675,7 @@ test_that('AD works with a model calling a custom dist and custom fun', {
     methods = list(
       derivsRun = function(x = double(1)) {
         ans <- nimDerivs(run(x), wrt = 1:length(x), order = 0:2,
-                         model = m, updateNodes = updateNodes, constantNodes = constantNodes)
+                         model = model, updateNodes = updateNodes, constantNodes = constantNodes)
         return(ans)
         returnType(ADNimbleList())
       }),
@@ -521,6 +695,151 @@ test_that('AD works with a model calling a custom dist and custom fun', {
   nim_expect_equal(Rans$value, Cans$value, RCrelTol0)
   nim_expect_equal(Rans$jacobian, Cans$jacobian, RCrelTol1)
   nim_expect_equal(Rans$hessian, Cans$hessian, RCrelTol2)
+  deregisterDistributions("dmynorm")
+})
+
+test_that('AD works with a model calling a user-defined distribution and user-defined function in a nimbleFunction object', {
+  stuff <- nimbleFunction(
+    setup=TRUE,
+    methods = list(
+      foo = function(x = double()) {
+        return(exp(x))
+        returnType(double())
+      },
+      d = function(x = double(), mu = double(), log = double(0, default = 0)) {
+        return(dnorm(x, mu, 1, log = log))
+        returnType(double())
+      }
+    ),
+    buildDerivs = c("foo", "d")
+  )
+  stuff1 <- stuff()
+  temporarilyAssignInGlobalEnv(stuff1)
+
+  m <- nimbleModel(
+    nimbleCode({
+      for(i in 1:3) {
+        a[i] ~ dbeta(2, 2)
+        x[i] ~ stuff1$d(mu = stuff1$foo(a[i]))
+      }
+    }),
+    inits = list(a = c(.1, .2, .3), x = c(.2, .3, .4)),
+    buildDerivs = TRUE
+  )
+  # temporarilyAssignInGlobalEnv(m)
+  temporarilyAssignInGlobalEnv(r_stuff1)
+
+  adfun <- nimbleFunction(
+    setup = function(model, wrt, calc) {
+      infoNodes <- makeModelDerivsInfo(model, wrtNodes = wrt, calcNodes = calc)
+      updateNodes <- infoNodes$updateNodes
+      constantNodes <- infoNodes$constantNodes
+    },
+    run = function(x = double(1)) {
+      values(model, wrt) <<- x
+      ans <- model$calculate(calc)
+      return(ans)
+      returnType(double())
+    },
+    methods = list(
+      derivsRun = function(x = double(1)) {
+        ans <- nimDerivs(run(x), wrt = 1:length(x), order = 0:2,
+                         model = model, updateNodes = updateNodes, constantNodes = constantNodes)
+        return(ans)
+        returnType(ADNimbleList())
+      }),
+    buildDerivs = 'run'
+  )
+  temporarilyAssignInGlobalEnv(adfun)
+
+  wrt <- 'a[1:3]'
+  Radfun1 <- adfun(m, wrt, m$getDependencies(wrt))
+  Cm <- compileNimble(m)
+  Cadfun1 <- compileNimble(Radfun1, project = m)
+  xRec <- c(.3, .6, .9)
+  xTest <- c(.8, .1, .4)
+  Rans <- Radfun1$derivsRun(xTest)
+  Cadfun1$derivsRun(xRec)
+  Cans <- Cadfun1$derivsRun(xTest)
+  nim_expect_equal(Rans$value, Cans$value, RCrelTol0)
+  nim_expect_equal(Rans$jacobian, Cans$jacobian, RCrelTol1)
+  nim_expect_equal(Rans$hessian, Cans$hessian, RCrelTol2)
+  deregisterDistributions("stuff1$d")
+})
+
+test_that('AD works with a model calling a user-defined distribution and user-defined function in a nimbleFunction object with double taping', {
+  stuffA <- nimbleFunction(
+    setup=TRUE,
+    methods = list(
+      foo = function(x = double()) {
+        return(exp(x))
+        returnType(double())
+      },
+      derivs_foo = function(x = double()) {
+        ans <- nimDerivs(foo(x), wrt = 1, order = 1)
+        res <- ans$jacobian[1,1]
+        return(res)
+        returnType(double())
+      },
+      d = function(x = double(), mu = double(), log = double(0, default = 0)) {
+        return(dnorm(x, mu, 1, log = log))
+        returnType(double())
+      }
+    ),
+    buildDerivs = c("foo", "d", "derivs_foo")
+  )
+  stuffA1 <- stuffA()
+  temporarilyAssignInGlobalEnv(stuffA1)
+
+  mA <- nimbleModel(
+    nimbleCode({
+      for(i in 1:3) {
+        a[i] ~ dbeta(2, 2)
+        x[i] ~ stuffA1$d(mu = stuffA1$foo(a[i]))
+      }
+    }),
+    inits = list(a = c(.1, .2, .3), x = c(.2, .3, .4)),
+    buildDerivs = TRUE
+  )
+  temporarilyAssignInGlobalEnv(mA)
+  temporarilyAssignInGlobalEnv(r_stuffA1)
+
+  adfunA <- nimbleFunction(
+    setup = function(model, wrt, calc) {
+      infoNodes <- makeModelDerivsInfo(model, wrtNodes = wrt, calcNodes = calc)
+      updateNodes <- infoNodes$updateNodes
+      constantNodes <- infoNodes$constantNodes
+    },
+    run = function(x = double(1)) {
+      values(model, wrt) <<- x
+      ans <- model$calculate(calc)
+      return(ans)
+      returnType(double())
+    },
+    methods = list(
+      derivsRun = function(x = double(1)) {
+        ans <- nimDerivs(run(x), wrt = 1:length(x), order = 0:2,
+                         model = model, updateNodes = updateNodes, constantNodes = constantNodes)
+        return(ans)
+        returnType(ADNimbleList())
+      }),
+    buildDerivs = 'run'
+  )
+  temporarilyAssignInGlobalEnv(adfunA)
+
+  wrt <- 'a[1:3]'
+  RadfunA1 <- adfunA(mA, wrt, mA$getDependencies(wrt))
+  CmA <- compileNimble(mA)
+  CadfunA1 <- compileNimble(RadfunA1, project = mA)
+  xRec <- c(.3, .6, .9)
+  xTest <- c(.8, .1, .4)
+  Rans <- RadfunA1$derivsRun(xTest)
+  CadfunA1$derivsRun(xRec)
+  Cans <- CadfunA1$derivsRun(xTest)
+  nim_expect_equal(Rans$value, Cans$value, RCrelTol0)
+  nim_expect_equal(Rans$jacobian, Cans$jacobian, RCrelTol1)
+  nim_expect_equal(Rans$hessian, Cans$hessian, RCrelTol2)
+  deregisterDistributions("stuffA1$d")
 })
 
 test_that('AD works with reset', {
@@ -609,7 +928,7 @@ test_that('AD works with a model with do_update', {
     }),
     buildDerivs = TRUE
   )
-  temporarilyAssignInGlobalEnv(m)
+  # temporarilyAssignInGlobalEnv(m)
 
   adfun <- nimbleFunction(
     setup = function(model, nodes) {
@@ -629,7 +948,7 @@ test_that('AD works with a model with do_update', {
                            reset = logical(0, default = FALSE)) {
         ans <- nimDerivs(run(x), wrt = 1:length(x), order = 0:2,
                          do_update = do_update, reset = reset,
-                         model = m, updateNodes = updateNodes)
+                         model = model, updateNodes = updateNodes)
         return(ans)
         returnType(ADNimbleList())
       }),
@@ -674,7 +993,7 @@ test_that('AD works with a model with do_update', {
     }),
     buildDerivs = TRUE
   )
-  temporarilyAssignInGlobalEnv(m)
+  # temporarilyAssignInGlobalEnv(m)
 
   adfun <- nimbleFunction(
     setup = function(model, nodes) {
@@ -694,7 +1013,7 @@ test_that('AD works with a model with do_update', {
                            reset = logical(0, default = FALSE)) {
         ans <- nimDerivs(run(x), wrt = 1:length(x), order = 0:2,
                          do_update = do_update, reset = reset,
-                         model = m, updateNodes = updateNodes)
+                         model = model, updateNodes = updateNodes)
         return(ans)
         returnType(ADNimbleList())
       }),
@@ -741,7 +1060,7 @@ test_that('AD works with a model with do_update in double-taping', {
     }),
     buildDerivs = TRUE
   )
-  temporarilyAssignInGlobalEnv(m)
+  # temporarilyAssignInGlobalEnv(m)
 
   adfun <- nimbleFunction(
     setup = function(model, nodes) {
@@ -761,7 +1080,7 @@ test_that('AD works with a model with do_update in double-taping', {
                            reset = logical(0, default = FALSE)) {
         ans <- nimDerivs(run(x), wrt = 1:length(x), order = 0,
                          do_update = do_update, reset = reset,
-                         model = m, updateNodes = updateNodes)
+                         model = model, updateNodes = updateNodes)
         return(ans$value[1])
         returnType(double())
       },
@@ -773,7 +1092,7 @@ test_that('AD works with a model with do_update in double-taping', {
         ans <- nimDerivs(valueRun(x, inner_do_update, inner_reset),
                          wrt = 1:length(x),
                          do_update = do_update, reset = reset,
-                         model = m, updateNodes = updateNodes)
+                         model = model, updateNodes = updateNodes)
         return(ans)
         returnType(ADNimbleList())
       }
@@ -906,7 +1225,7 @@ test_that('non-AD function in AD-enabled model', {
 })
 
 
-test_that("derivatives work if a non-supported user-defined call is not included", {
+test_that("derivatives work if a non-supported user-defined distribution is not included", {
   dfoo2 <- nimbleFunction(
     run = function(x = double(), k = double(), log = integer(0, default = 0)) {
       return(-k * x)
@@ -941,7 +1260,7 @@ test_that("derivatives work if a non-supported user-defined call is not included
   expect_equal( round(ans$jacobian, digits = 2), matrix(-1.5) )
 })
 
-test_that("C++ run-time error if non-supported user-defined call is included in model$calculate", {
+test_that("C++ run-time error if non-supported user-defined distribution is included in model$calculate", {
   dfoo3 <- nimbleFunction(
     run = function(x = double(), k = double(), log = integer(0, default = 0)) {
       return(-k * x)
@@ -973,9 +1292,10 @@ test_that("C++ run-time error if non-supported user-defined call is included in
   test1 <- test(m, c("a", "z"))
   ctest1 <- compileNimble(test1, project = m)
   expect_error(ans <- ctest1$run(), "a model node is being included in derivatives that does not support them")
+  deregisterDistributions("dfoo3")
 })
 
-test_that("C++ run-time error if non-supported user-defined call is included in model$calculate", {
+test_that("C++ run-time error if non-supported distribution is included in model$calculate", {
   mc <- nimbleCode({
     a ~ dnorm(0, 1)
     z ~ dconstraint(a > 1)
@@ -1000,7 +1320,7 @@ test_that("C++ run-time error if non-supported user-defined call is included in
 })
 
 
-test_that("warn if non-supported user-defined call is included in model$calculate", {
+test_that("warn if non-supported distribution is included in model$calculate", {
     nimbleOptions(unsupportedDerivativeHandling='warn')
     mc <- nimbleCode({
         a ~ dnorm(0, 1)
