@@ -5,16 +5,14 @@ options(warn = 1)
 nimbleVerboseSetting <- nimbleOptions('verbose')
 nimbleOptions(verbose = FALSE)
 
-nimbleOptions(enableModelMacros = TRUE)
+nimbleOptions(enableMacros = TRUE)
 nimbleOptions(enableMacroComments = FALSE)
 
-context("Testing model macros")
-
 test_that('Macro expansion 1',
 {
     ## This converts a ~ testMacro(b)
     ## to inputs as (stoch = TRUE, LHS = a, b)
-    testMacro <- nimble:::model_macro_builder(
+    testMacro <- nimble:::buildMacro(
         function(stoch, LHS, RHSarg, modelInfo, .env) {
             ans <- substitute(
                 OP(LHS, dnorm(RHSarg, 1)),
@@ -26,12 +24,17 @@ test_that('Macro expansion 1',
         use3pieces = TRUE, ## default
         unpackArgs = TRUE  ## default
     )
-    temporarilyAssignInGlobalEnv(testMacro)
+ #   temporarilyAssignInGlobalEnv(testMacro)
     
-    expect_identical(
-        nimble:::processMacrosInternal(
+    process_macro <- nimble:::processMacrosInternal(
             quote(x[1] ~ testMacro(y[1])),
-        modelInfo = list())$code,
+        modelInfo = list(nMacros=0))
+
+    # Check that nMacros incremented
+    expect_equal(process_macro$modelInfo$nMacros, 1)
+
+    expect_identical(
+        process_macro$code,
         quote(x[1] ~ dnorm(y[1], 1))
     )
     
@@ -73,11 +76,67 @@ test_that('Macro expansion 1',
 })
 
 
+test_that('Macro expansion 1 with expression to get macro',
+{
+    ## This converts a ~ testMacro(b)
+    ## to inputs as (stoch = TRUE, LHS = a, b)
+  my_env <- new.env()
+  testMacro <- nimble:::buildMacro(
+        function(stoch, LHS, RHSarg, modelInfo, .env) {
+            ans <- substitute(
+                OP(LHS, dnorm(RHSarg, 1)),
+                list(LHS = LHS, RHSarg = RHSarg,
+                     OP = if(stoch) as.name('~') else as.name('<-'))
+            )
+            list(code = ans, modelInfo=modelInfo)
+        },
+        use3pieces = TRUE, ## default
+        unpackArgs = TRUE  ## default
+        )
+  my_env$testMacro <- testMacro
+  rm(testMacro)
+ #   temporarilyAssignInGlobalEnv(testMacro)
+
+    expect_identical(
+        nimble:::processMacrosInternal(
+            quote(x[1] ~ my_env$testMacro(y[1])),
+        modelInfo = list())$code,
+        quote(x[1] ~ dnorm(y[1], 1))
+    )
+
+    model1 <- nimbleModel(
+        nimbleCode({
+            x[1] ~ my_env$testMacro(y[1])
+        })
+    )
+    expect_identical(
+        model1$getCode()[[2]],
+        quote(x[1] ~ dnorm(y[1], 1))
+    )
+})
+
+
+test_that('Macro expansion 1 with namespace resolution checks but continues',
+{
+  # I don't really have a good way to check namespace resolution, but I will at least code it
+  # and manually go inside and see that it is being evaluated.
+  # The result of not finding a macro should be to return the code unmodified.
+  # nimble::nimbleFunction is just an arbitrary non-macro object in a namescape
+  # so it can be looked up and passed over as a non-macro
+    expect_identical(
+        nimble:::processMacrosInternal(
+            quote(x[1] ~ nimble::nimbleFunction(y[1])),
+            modelInfo = list())$code,
+        quote(x[1] ~ nimble::nimbleFunction(y[1]))
+    )
+})
+
+
 test_that('Macro expansion 2',
 {
     ## Like testMacro above but with unpackArgs = FALSE.
     ## This takes a ~ testMacro(b) with arguments (stoch = TRUE, LHS = a, RHS = b)
-    testMacro <- nimble:::model_macro_builder(
+    testMacro <- nimble:::buildMacro(
         function(stoch, LHS, RHS, modelInfo, .env) {
             if(RHS[[1]] != "testMacro")
                 stop("Problem with how testMacro was called")
@@ -93,7 +152,7 @@ test_that('Macro expansion 2',
         unpackArgs = FALSE  ## NON-default
     )
     
-    temporarilyAssignInGlobalEnv(testMacro)
+#    temporarilyAssignInGlobalEnv(testMacro)
 
     expect_identical(
         nimble:::processMacrosInternal(
@@ -119,7 +178,7 @@ test_that('Macro expansion 3',
     ## Like expansion 1, but with multiple named arguments.
     ## This takes a ~ testMacro(newVar = b, newIndex = 3)
     ## as arguments (stoch = TRUE, LHS = a, newVar = b, newIndex = 3)
-    testMacro <- nimble:::model_macro_builder(
+    testMacro <- nimble:::buildMacro(
         function(stoch, LHS, newIndex, newVar, modelInfo, .env) {
         RHS <- substitute(
             X[I],
@@ -137,7 +196,7 @@ test_that('Macro expansion 3',
         unpackArgs = TRUE  ## default
     )
 
-    temporarilyAssignInGlobalEnv(testMacro)
+#    temporarilyAssignInGlobalEnv(testMacro)
     
     expect_identical(
         nimble:::processMacrosInternal(
@@ -164,7 +223,7 @@ test_that('Macro expansion 4',
     ## This takes a ~ testMacro(b) as a single argument,
     ## (code = a ~ testMacro(b)).
     ## It does not even need to be in a line with '~' or '<-'.
-    testMacro <- nimble:::model_macro_builder(
+    testMacro <- nimble:::buildMacro(
         function(code, modelInfo, .env) {
             code[[3]][[1]] <- as.name('dnorm')
             list(code = code, modelInfo=modelInfo)
@@ -173,7 +232,7 @@ test_that('Macro expansion 4',
         unpackArgs = FALSE  ## NON-default
     )
 
-    temporarilyAssignInGlobalEnv(testMacro)
+#    temporarilyAssignInGlobalEnv(testMacro)
 
     expect_identical(
         nimble:::processMacrosInternal(
@@ -200,7 +259,7 @@ test_that('Macro expansion 5',
     ## This takes a line of code split into arguments.
     ## It is designed for a line without '~' or '<-'.
     ## It takes testMacro(arg1, arg2, arg3)
-    testMacro <- nimble:::model_macro_builder(
+    testMacro <- nimble:::buildMacro(
         function(arg1, arg2, arg3, modelInfo, .env) {
             code <- substitute(A <- B + C, list(A = arg1, B = arg2, C = arg3))
             list(code = code, modelInfo = modelInfo)
@@ -209,7 +268,7 @@ test_that('Macro expansion 5',
         unpackArgs = TRUE  ## default
     )
 
-    temporarilyAssignInGlobalEnv(testMacro)
+#    temporarilyAssignInGlobalEnv(testMacro)
     
     expect_identical(
         nimble:::processMacrosInternal(
@@ -232,7 +291,7 @@ test_that('Macro expansion 5',
 
 test_that('Macro expansion 6 (recursive macro expansion)',
 {
-    testMacroInner <- nimble:::model_macro_builder(
+    testMacroInner <- nimble:::buildMacro(
         function(stoch, LHS, RHSarg, modelInfo, ...) {
             ans <- substitute(
                 OP(LHS, dnorm(RHSarg, 1)),
@@ -244,11 +303,11 @@ test_that('Macro expansion 6 (recursive macro expansion)',
         use3pieces = TRUE, ## default
         unpackArgs = TRUE  ## default
     )
-    temporarilyAssignInGlobalEnv(testMacroInner)
+#    temporarilyAssignInGlobalEnv(testMacroInner)
     
     ## a ~ testMacroOuter(b)
     ## becomes a ~ testMacroInner(b)
-    testMacroOuter <- nimble:::model_macro_builder(
+    testMacroOuter <- nimble:::buildMacro(
         function(code, modelInfo, ...) {
             code[[3]][[1]] <- as.name('testMacroInner')
             list(code = code, modelInfo = modelInfo)
@@ -256,7 +315,7 @@ test_that('Macro expansion 6 (recursive macro expansion)',
         use3pieces = FALSE, ## NON-default
         unpackArgs = FALSE  ## NON-default
     )
-    temporarilyAssignInGlobalEnv(testMacroOuter)
+#    temporarilyAssignInGlobalEnv(testMacroOuter)
     
     expect_identical(
         nimble:::processMacrosInternal(
@@ -279,6 +338,59 @@ test_that('Macro expansion 6 (recursive macro expansion)',
     )
 })
 
+
+test_that('Macro expansion 6b (recursive macro expansion with calls to find macros)',
+{
+  inner.env <- new.env()
+  inner.env$testMacroInner <- nimble:::buildMacro(
+        function(stoch, LHS, RHSarg, modelInfo, ...) {
+            ans <- substitute(
+                OP(LHS, dnorm(RHSarg, 1)),
+                list(LHS = LHS, RHSarg = RHSarg,
+                     OP = if(stoch) as.name('~') else as.name('<-'))
+            )
+            list(code = ans, modelInfo = modelInfo)
+        },
+        use3pieces = TRUE, ## default
+        unpackArgs = TRUE  ## default
+    )
+#    temporarilyAssignInGlobalEnv(testMacroInner)
+
+    ## a ~ testMacroOuter(b)
+    ## becomes a ~ testMacroInner(b)
+  outer.env <- new.env()
+  outer.env$testMacroOuter <- nimble:::buildMacro(
+        function(code, modelInfo, ...) {
+            code[[3]][[1]] <- quote(inner.env$testMacroInner) #as.name('testMacroInner')
+            list(code = code, modelInfo = modelInfo)
+        },
+        use3pieces = FALSE, ## NON-default
+        unpackArgs = FALSE  ## NON-default
+    )
+#    temporarilyAssignInGlobalEnv(testMacroOuter)
+
+    expect_identical(
+        nimble:::processMacrosInternal(
+            quote(x[1] ~ outer.env$testMacroOuter(y[1])),
+            modelInfo = list()
+        )$code,
+        quote(x[1] ~ dnorm(y[1], 1))
+    )
+
+
+    model <- nimbleModel(
+        nimbleCode({
+            x[1] ~ outer.env$testMacroOuter(y[1])
+        })
+    )
+
+    expect_identical(
+        model$getCode()[[2]],
+        quote(x[1] ~ dnorm(y[1], 1))
+    )
+})
+
+
 test_that(paste0('Macro expansion 7 (correct trapping of ',
                  'failure in recursive macro expansion)'),
 {
@@ -286,7 +398,7 @@ test_that(paste0('Macro expansion 7 (correct trapping of ',
     ## test of recursive expansion:
     ## This a ~ testMacroInner(b)
     ## with inputs as (stoch = TRUE, LHS = a, b)
-    testMacroInner <- nimble:::model_macro_builder(
+    testMacroInner <- nimble:::buildMacro(
         function(stoch, LHS, RHSarg, modelInfo, ...) {
             stop()
             ans <- substitute(
@@ -299,10 +411,10 @@ test_that(paste0('Macro expansion 7 (correct trapping of ',
         use3pieces = TRUE, ## default
         unpackArgs = TRUE  ## default
     )
-    temporarilyAssignInGlobalEnv(testMacroInner)
+#    temporarilyAssignInGlobalEnv(testMacroInner)
     ## a ~ testMacroOuter(b)
     ## becomes a ~ testMacroInner(b)
-    testMacroOuter <- nimble:::model_macro_builder(
+    testMacroOuter <- nimble:::buildMacro(
         function(code, modelInfo, .env) {
             code[[3]][[1]] <- as.name('testMacroInner')
             list(code = code, modelInfo = modelInfo)
@@ -310,7 +422,7 @@ test_that(paste0('Macro expansion 7 (correct trapping of ',
         use3pieces = FALSE, ## NON-default
         unpackArgs = FALSE  ## NON-default
     )
-    temporarilyAssignInGlobalEnv(testMacroOuter)
+#    temporarilyAssignInGlobalEnv(testMacroOuter)
 
     #cat("\nTwo 'unused argument' error messages known to occur here:\n") 
     #cat("\nNot sure why these are supposed to fail:\n")
@@ -360,7 +472,7 @@ test_that(paste0('Macro expansion 7 (correct trapping of ',
 test_that('duplicate variables from macro expansion error-trapped correctly',
 {
     ## from roxygen example
-    flat_normal_priors <- nimble:::model_macro_builder(
+    flat_normal_priors <- nimble:::buildMacro(
         function(..., modelInfo) {
           allVars <- list(...)
           allVars <- allVars[ !(names(allVars) %in% (c('.constants','.env'))) ]
@@ -375,7 +487,7 @@ test_that('duplicate variables from macro expansion error-trapped correctly',
         use3pieces = FALSE,
         unpackArgs = TRUE
     )
-    temporarilyAssignInGlobalEnv(flat_normal_priors)
+    # temporarilyAssignInGlobalEnv(flat_normal_priors)
     ## Safe use of try due to immediate next test
     expect_error(model <- nimbleModel(
         nimbleCode(
@@ -388,7 +500,7 @@ test_that('duplicate variables from macro expansion error-trapped correctly',
 
 test_that('duplicate nested indices from macro expansion error-trapped correctly',
 {
-    all_dnorm <- nimble:::model_macro_builder(
+    all_dnorm <- nimble:::buildMacro(
         function(stoch, LHS, RHSvar, start, end, sd = 1, modelInfo, ...) {
             newCode <- substitute(
                 for(i in START:END) {
@@ -404,7 +516,7 @@ test_that('duplicate nested indices from macro expansion error-trapped correctly
         use3pieces = TRUE,
         unpackArgs = TRUE 
     )
-    temporarilyAssignInGlobalEnv(all_dnorm)
+#    temporarilyAssignInGlobalEnv(all_dnorm)
     ## Safe use of try due to immediately next test
     expect_error(model <- nimbleModel(
         nimbleCode(
@@ -424,7 +536,7 @@ test_that('duplicate nested indices from macro expansion error-trapped correctly
         }
         )), regexp = 'Stopped after processing macros and if/then/else statements')
     )
-    expect_true(grepl("dnorm(mu[i], 1)", out[2], fixed=TRUE)) 
+    expect_true(grepl("dnorm(mu[i], 1)", out[3], fixed=TRUE)) 
 
     nimbleOptions(stop_after_processing_model_code = FALSE)
 })
@@ -433,9 +545,9 @@ test_that('constants and env are accessed correctly, even in recursion',
 {
   testEnv <- new.env()
   testEnv$var_in_env <- 5
-  temporarilyAssignInGlobalEnv(testEnv)
+  # temporarilyAssignInGlobalEnv(testEnv)
 
-  testMacroInner <- nimble:::model_macro_builder(
+  testMacroInner <- nimble:::buildMacro(
     function(stoch, LHS, RHSarg, modelInfo, .env) {
       constant_of_interest <- modelInfo$constants$constant_of_interest
       var_in_env <- eval(quote(var_in_env), envir = .env)
@@ -451,12 +563,12 @@ test_that('constants and env are accessed correctly, even in recursion',
         use3pieces = TRUE, ## default
         unpackArgs = TRUE  ## default
         )
-  temporarilyAssignInGlobalEnv(testMacroInner)
+  # temporarilyAssignInGlobalEnv(testMacroInner)
 
 
     ## a ~ testMacroOuter(b)
     ## becomes a ~ testMacroInner(b)
-  testMacroOuter <- nimble:::model_macro_builder(
+  testMacroOuter <- nimble:::buildMacro(
         function(code, modelInfo, .env) {
             code[[3]][[1]] <- as.name('testMacroInner')
             list(code = code, modelInfo = modelInfo)
@@ -464,7 +576,7 @@ test_that('constants and env are accessed correctly, even in recursion',
         use3pieces = FALSE, ## NON-default
         unpackArgs = FALSE  ## NON-default
         )
-  temporarilyAssignInGlobalEnv(testMacroOuter)
+#  temporarilyAssignInGlobalEnv(testMacroOuter)
 
     macroOut <- nimble:::processMacrosInternal(
                    quote(x[1] ~ testMacroOuter(y[1])),
@@ -503,7 +615,7 @@ test_that("macros can add constants", {
   )
   class(testMacro) <- "model_macro"
 
-
+  # needed since this test creates macro manually
   temporarilyAssignInGlobalEnv(testMacro)
 
   code <- nimbleCode({
@@ -529,9 +641,20 @@ test_that("macros can add constants", {
 test_that("codeProcessModelMacros converts factors to numeric", {
   inp_constants <- list(y = rnorm(3), x = factor(c("a","b","c")))
 
+  testMacro <- list(process = function(code, modelInfo, .env){
+      code[[3]] <- quote(dnorm(0, sd = 1))
+      modelInfo$constants$b <- 2
+      list(code = code, modelInfo=modelInfo)
+    }
+  )
+  class(testMacro) <- "model_macro"
+
+  # needed since this test creates macro manually
+  temporarilyAssignInGlobalEnv(testMacro)
+
   code <- nimbleCode({
     for (i in 1:3){
-      y[i] ~ dnorm(x[i], sd = 1)
+      y[i] ~ testMacro()
     }
   })
   
@@ -542,6 +665,20 @@ test_that("codeProcessModelMacros converts factors to numeric", {
     as.numeric(inp_constants$x)
   )
 
+  # In this case there is no macro in the code, so the constants should not
+  # be automatically converted from factor to numeric
+  code <- nimbleCode({
+    for (i in 1:3){
+      y[i] ~ dnorm(x[i], sd = 1)
+    }
+  })
+  
+  mod <- nimble:::codeProcessModelMacros(code, modelInfo=list(constants=inp_constants), env=parent.frame())
+
+  expect_equal(
+    mod$modelInfo$constants$x,
+    inp_constants$x
+  )
 })
 
 test_that("convertFactorConstantsToNumeric converts factors to numeric", {
@@ -693,7 +830,7 @@ test_that("nimbleModel creates and stores list of macro-generated parameters", {
   )
   class(testMacro) <- "model_macro"
 
-  temporarilyAssignInGlobalEnv(testMacro)
+  # temporarilyAssignInGlobalEnv(testMacro)
 
   code <- nimbleCode({
     x ~ testMacro()
@@ -720,7 +857,7 @@ test_that("getMacroParameters() can pull out different subsets of parameters", {
   )
   class(testMacro) <- "model_macro"
 
-  temporarilyAssignInGlobalEnv(testMacro)
+  # temporarilyAssignInGlobalEnv(testMacro)
 
   code <- nimbleCode({
     y[1:n] ~ testMacro()
@@ -797,6 +934,147 @@ test_that("removeExtraBrackets cleans up output from processMacrosInternal",{
     })
   )
 
+  # Check handling of for loops without { (issue #1503)
+  code = nimbleCode({
+                  for(j in 1:3)   # NOTE no bracket
+                    for(l in 1:2) {
+                    y[i,j,l] ~ dnorm(mu[1+k1[i], 2, k2[i]], var = s0)
+                    mu[i, j, l] ~ dnorm(0, 1)
+                }
+       })
+  expect_equal(
+    nimble:::removeExtraBrackets(code),
+    quote({
+    for (j in 1:3) for (l in 1:2) {
+        y[i, j, l] ~ dnorm(mu[1 + k1[i], 2, k2[i]], var = s0)
+        mu[i, j, l] ~ dnorm(0, 1)
+    }
+    })
+  )
+
+  code <- nimbleCode({
+    for (i in 1:2)
+      for (j in 1:3)
+        for (l in 1:2)
+          y[i,j,l] ~ dnorm()
+  })
+  expect_equal(
+    nimble:::removeExtraBrackets(code),
+    quote({
+      for (i in 1:2) for (j in 1:3) for (l in 1:2) y[i, j, l] ~ dnorm()
+    })
+  )
+
+  code <- nimbleCode({
+    for (i in 1:2) for (j in 1:3) for (l in 1:2) y[i,j,l] ~ dnorm()
+  })
+  expect_equal(
+    nimble:::removeExtraBrackets(code),
+    quote({
+      for (i in 1:2) for (j in 1:3) for (l in 1:2) y[i, j, l] ~ dnorm()
+    })
+  )
+
+  code <- nimbleCode({
+    for (i in 1:2) y[i] ~ dnorm()
+  })
+  expect_equal(
+    nimble:::removeExtraBrackets(code),
+    quote({
+      for (i in 1:2) y[i] ~ dnorm()
+    })
+  )
+
+  code = nimbleCode({
+                  {
+                  for(j in 1:3)   # NOTE no bracket
+                    for(l in 1:2) {
+                    y[i,j,l] ~ dnorm(mu[1+k1[i], 2, k2[i]], var = s0)
+                    { # random extra brackets
+                      y[1] <- 3 
+                    }
+                    mu[i, j, l] ~ dnorm(0, 1)
+                }
+                }
+       })
+  expect_equal(
+    nimble:::removeExtraBrackets(code),
+    quote({
+    for (j in 1:3) for (l in 1:2) {
+        y[i, j, l] ~ dnorm(mu[1 + k1[i], 2, k2[i]], var = s0)
+        y[1] <- 3
+        mu[i, j, l] ~ dnorm(0, 1)
+    }
+    })
+  )
+
+  # Check that removeExtraBrackets is applied correctly by codeProcessModelMacros
+  testMacro <- list(process = function(code, modelInfo, .env){
+      code[[3]] <- quote(dnorm(0, sd = 1))
+      modelInfo$constants$b <- 2
+      list(code = code, modelInfo=modelInfo)
+    }
+  )
+  class(testMacro) <- "model_macro"
+
+  # needed since this test creates macro manually
+  temporarilyAssignInGlobalEnv(testMacro)
+
+  code <- nimbleCode({
+  {
+    alpha <- 1
+    for (i in 1:n){
+      {
+        for (j in 1:k){
+          z <- testMacro()
+          {
+          y <- 1
+          }
+        }
+      }
+    }
+  }
+  })
+
+  minfo <- list()
+  
+  expect_equal(
+    nimble:::codeProcessModelMacros(code, minfo, env=environment())$code,
+    quote({
+      alpha <- 1
+      for (i in 1:n){
+        for (j in 1:k){
+          z <- dnorm(0, sd = 1)
+          y <- 1
+        }
+      }
+    })
+  )
+
+  # Now make sure removeExtraBrackets is NOT applied when there is no
+  # macro in the code
+  code <- nimbleCode({
+  {
+    alpha <- 1
+    for (i in 1:n){
+      {
+        for (j in 1:k){
+          z <- 1
+          {
+          y <- 1
+          }
+        }
+      }
+    }
+  }
+  })
+
+  minfo <- list()
+  
+  expect_equal(
+    nimble:::codeProcessModelMacros(code, minfo, env=environment())$code,
+    code)
+
 })
 
 test_that("comments are added to macro output if enableMacroComments = TRUE",{
@@ -815,14 +1093,14 @@ test_that("comments are added to macro output if enableMacroComments = TRUE",{
     }
   )
   class(testMacro) <- "model_macro"
-  temporarilyAssignInGlobalEnv(testMacro)
+  # temporarilyAssignInGlobalEnv(testMacro)
   
   testMacroInner <- list(process = function(code, modelInfo, .env){
     code[[3]] <- quote(alpha + beta)
     list(code = code, modelInfo = modelInfo)
   })
   class(testMacroInner) <- "model_macro"
-  temporarilyAssignInGlobalEnv(testMacroInner)
+  # temporarilyAssignInGlobalEnv(testMacroInner)
 
   code <- nimbleCode({
     y[1:n] ~ testMacro()
@@ -931,7 +1209,7 @@ test_that("macros can generate inits and these are used by models",{
       }
     )
   class(testMacro) <- "model_macro"
-  temporarilyAssignInGlobalEnv(testMacro)
+  # temporarilyAssignInGlobalEnv(testMacro)
 
   code <- nimbleCode({
     x ~ testMacro()
@@ -953,7 +1231,7 @@ test_that("macros can generate inits and these are used by models",{
       }
     )
   class(testMacroInits) <- "model_macro"
-  temporarilyAssignInGlobalEnv(testMacroInits)
+  # temporarilyAssignInGlobalEnv(testMacroInits)
 
   code <- nimbleCode({
     x ~ testMacroInits()
@@ -980,7 +1258,7 @@ test_that("message given if same parameter is generated by macros multiple times
     }
   )
   class(testMacro) <- "model_macro"
-  temporarilyAssignInGlobalEnv(testMacro)
+  # temporarilyAssignInGlobalEnv(testMacro)
   
   code <- nimbleCode({
     testMacro()
@@ -996,4 +1274,4 @@ test_that("message given if same parameter is generated by macros multiple times
 options(warn = RwarnLevel)
 nimbleOptions(verbose = nimbleVerboseSetting)
 
-nimbleOptions(enableModelMacros = FALSE)
+nimbleOptions(enableMacros = FALSE)
